/*
We use VID/PID 0x046D/0xC00E which is taken from a Logitech mouse. Don't
publish any hardware using these IDs! This is for demonstration only!
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[52] = { /* USB report descriptor, size must match usbconfig.h */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xA1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM
    0x29, 0x03,                    //     USAGE_MAXIMUM
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Const,Var,Abs)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x38,                    //     USAGE (Wheel)
    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
    0x25, 0x7F,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //     REPORT_SIZE (8)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x81, 0x06,                    //     INPUT (Data,Var,Rel)
    0xC0,                          //   END_COLLECTION
    0xC0,                          // END COLLECTION
};
/* This is the same report descriptor as seen in a Logitech mouse. The data
 * described by this descriptor consists of 4 bytes:
 *      .  .  .  .  . B2 B1 B0 .... one byte with mouse button states
 *     X7 X6 X5 X4 X3 X2 X1 X0 .... 8 bit signed relative coordinate x
 *     Y7 Y6 Y5 Y4 Y3 Y2 Y1 Y0 .... 8 bit signed relative coordinate y
 *     W7 W6 W5 W4 W3 W2 W1 W0 .... 8 bit signed relative coordinate wheel
 */
typedef struct{
    uchar   buttonMask;
    char    dx;
    char    dy;
    char    dWheel;
}report_t;

static report_t reportBuffer;

static uchar    idleRate;   /* repeat rate for keyboards, never used for mice */

#define uchar unsigned char
#define uint unsigned int

// Pin IR
#define	xRC5_IN		PIND
#define	xRC5		PD1						// IR input low active

// PIN LED
#define LED_DDR		DDRD
#define LED_PORT	PORTD
#define LED_PIN     PD4

#ifndef F_CPU
#define	F_CPU		12.0000e6
#endif

#define RC5TIME 	1.778e-3					// 1.778msec
#define PULSE_MIN	(uchar)(F_CPU / 512 * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2	(uchar)(F_CPU / 512 * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX	(uchar)(F_CPU / 512 * RC5TIME * 1.2 + 0.5)


volatile uchar	rc5_bit;						// bit value
volatile uchar	rc5_time;						// count bit time
volatile uint	rc5_tmp;						// shift bits in
volatile uint	rc5_data;						// store result

//
// ISR fuer RC5-IR-Empfaenger
//
ISR(TIMER0_OVF_vect)
{
	uint tmp = rc5_tmp;							// for faster access

	TCNT0 = -2;									// 2 * 256 = 512 cycle

	if( ++rc5_time > PULSE_MAX ){				// count pulse time
		if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
			rc5_data = tmp;
		tmp = 0;
	}

	if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
		rc5_bit = ~rc5_bit;						// 0x00 -> 0xFF -> 0x00

		if( rc5_time < PULSE_MIN )				// to short
			tmp = 0;

		if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
			if( !(tmp & 0x4000) )				// not to many bits
				tmp <<= 1;						// shift
			if( !(rc5_bit & 1<<xRC5) )			// inverted bit
				tmp |= 1;						// insert new bit
			rc5_time = 0;						// count next pulse time
		}
	}

	rc5_tmp = tmp;
}


/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    /* The following requests are never used. But since they are required by
     * the specification, we implement them in this example.
     */
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            usbMsgPtr = (void *)&reportBuffer;
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{

	uchar   i;

	uint	temp_key;
	uint	rc5_key_code;
	uint	rc5_toggle_bit;
	uint	toggle_bit_old = 0;
	uchar	diff = 5;
	uchar	post_send = 0;

	TCCR0 = 1<<CS02;							// divide by 256
	TIMSK = 1<<TOIE0;							// enable timer interrupt

	LED_DDR   =  (1 << LED_PIN);                // LED-Pin
	LED_PORT &= ~(1 << LED_PIN);                // LED off

    wdt_enable(WDTO_1S);

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */

        wdt_reset();        // watchdog refresh

		cli();              // rc5 
		temp_key = rc5_data;			
		rc5_data = 0;
		sei();

        usbPoll();
		if (temp_key) {									// rc5

			rc5_toggle_bit = temp_key >> 11 & 1;		// toggle bit
			rc5_key_code   = ((temp_key & 0x3F) | (~temp_key >> 7 & 0x40));	// key code
			post_send = 1;

			switch (rc5_key_code) {

				case 1 ... 9:
							diff = (char)rc5_key_code;
							break;
				case 17:	reportBuffer.dy = diff; //17
							break;
				case 16:	reportBuffer.dy = 0 - diff; //16
							break;
				case 22: 	reportBuffer.dx = diff; //22
							break;
				case 21: 	reportBuffer.dx = 0 - diff; //21
							break;
				case 33: 	if (rc5_toggle_bit != toggle_bit_old) { //33
								reportBuffer.dWheel = diff;		
							}
							break;
				case 32: 	if (rc5_toggle_bit != toggle_bit_old) { //32
								reportBuffer.dWheel = 0 - diff;	
							}
							break;
				case 35: 	if (rc5_toggle_bit != toggle_bit_old) {
								reportBuffer.buttonMask = 1;			// linke Maustaste (35)
							}
							break;
				case 18: 	if (rc5_toggle_bit != toggle_bit_old) {
								reportBuffer.buttonMask = 4;			// mittlere Maustaste (18)
							}
							break;
				case 12: 	if (rc5_toggle_bit != toggle_bit_old) {
								reportBuffer.buttonMask = 2;			// rechte Maustaste (12)
							}
							break;
			}
			toggle_bit_old = rc5_toggle_bit;            // Zustand Toggle-Bit merken

		    if (usbInterruptIsReady()){
		            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
					LED_PORT ^=(1 << LED_PIN);
					reportBuffer.dx = 0;
					reportBuffer.dy = 0;
					reportBuffer.dWheel = 0;
					reportBuffer.buttonMask = 0;
	        }
		} else    {
			if (post_send) {					// dies scheint wohl notwendig zu sein...,
				reportBuffer.dx = 0;			// zumindestens fuer die Maustasten
				reportBuffer.dy = 0;			// muss nach dem Setzen auch wieder
				reportBuffer.buttonMask = 0;	// Null gesendet werden...
				reportBuffer.dWheel = 0;
				if (usbInterruptIsReady()){
					usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
					post_send = 0;
				}
			}
		}
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
