/************************************************************************/
/*                                                                      */
/*                      RC5 Remote Receiver                             */
/*                      ===================                             */
/*                                                                      */
/* Schaltet via RC5-IR-Fernbedienung eine LED.                          */
/* Key-Code=1 --> LED an                                                */
/* Key-Code=2 --> LED aus                                               */
/* Key-Code=3 --> LED invers                                            */
/*                                                                      */
/* Hardwaregegebenheiten sind via Defines anzupassen...                 */
/*                                                                      */
/* Idee der RC5-Routine: Peter Dannegger (danni@specs.de)               */
/* Diese Routine kommt ohne externen Interrupt (INT0/1) aus. Desweiteren*/
/* wird nur ein fortlaufender Timerinterrupt verwendet, der auch fuer   */
/* andere Zwecke verwendet werden kann.                                 */
/*                                                                      */
/************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>

#define uchar unsigned char
#define uint unsigned int

// MC-Pin IR-Empfaenger
#define	xRC5_IN		PIND
#define	xRC5		PD1						// IR input low active

// MC-PIN LED
#define LED_DDR		DDRD
#define LED_PORT	PORTD
#define LED_PIN     PD4

#ifndef F_CPU
#define	F_CPU		16.0000e6
#endif

#define RC5TIME 	1.778e-3					// 1.778msec
#define PULSE_MIN	(uchar)(F_CPU / 512 * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2	(uchar)(F_CPU / 512 * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX	(uchar)(F_CPU / 512 * RC5TIME * 1.2 + 0.5)


volatile uchar	rc5_bit;						// bit value
volatile uchar	rc5_time;						// count bit time
volatile uint	rc5_tmp;						// shift bits in
volatile uint	rc5_data;						// store result


ISR(TIMER0_OVF_vect)
{
	uint tmp = rc5_tmp;							// for faster access

	TCNT0 = -2;									// 2 * 256 = 512 cycle

	if( ++rc5_time > PULSE_MAX ){				// count pulse time
		if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
			rc5_data = tmp;
		tmp = 0;
	}

	if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
		rc5_bit = ~rc5_bit;						// 0x00 -> 0xFF -> 0x00

		if( rc5_time < PULSE_MIN )				// to short
			tmp = 0;

		if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
			if( !(tmp & 0x4000) )				// not to many bits
				tmp <<= 1;						// shift
			if( !(rc5_bit & 1<<xRC5) )			// inverted bit
				tmp |= 1;						// insert new bit
			rc5_time = 0;						// count next pulse time
		}
	}

	rc5_tmp = tmp;
}

int main( void )
{
	uint i;
	uint rc5_key_code;
	uint rc5_device;
	uint rc5_toggle_bit;
	uint toggle_bit_old=0;

	TCCR0 = 1<<CS02;							// divide by 256
	TIMSK = 1<<TOIE0;							// enable timer interrupt

	LED_DDR   =  (1 << LED_PIN);                // LED-Pin als Ausgang
	LED_PORT &= ~(1 << LED_PIN);                // LED definiert auf aus

	sei();

	for(;;){									// main loop
		cli();
		i = rc5_data;							// read two bytes from interrupt !
		rc5_data = 0;
		sei();
		if (i){
			rc5_device     = i >> 6 & 0x1F;						// decice
			rc5_toggle_bit = i >> 11 & 1;						// toggle bit
			rc5_key_code   = ((i & 0x3F) | (~i >> 7 & 0x40));	// key code

			switch (rc5_key_code) {

				case 1:	LED_PORT |= (1 << LED_PIN);		// LED an
						break;
				case 2: LED_PORT &= ~(1 << LED_PIN);	// LED aus
						break;
				case 3: if (rc5_toggle_bit != toggle_bit_old) {	// wenn nicht Dauerdruecken...
							LED_PORT ^=(1 << LED_PIN);	// LED invertieren
						}
						break;
			}
			toggle_bit_old = rc5_toggle_bit;            // Zustand Toggle-Bit merken
		}
	}
}

